/************************************************************************
	BLOCKHED.PRO
 

                                                            
                          B L O C K H E A D                 
                                  A                          
                conversational program in PDC-PROLOG        
                            commanding                      
                             a robot                        
                        @ Henrik Prebensen,                 
                       Centre for Informatics               
                         Faculty of Letters                 
                      University of Copenhagen              
                          Version 28.01.88                  
                                                            
                                                            
                         System: 7 files,                   
                             blockhed.pro,                  
                             lexicon.blk,                   
                             control.blk                    
                             world.blk,                     
                             io.blk,                        
                             color.blk,                     
                             nocolor.blk.                   


	Copyright (c) 1988, 1990 by Prolog Development Center             
**************************************************************************/



                    /*::::::::::::::::::::::::::*/
                      /************************/
                      /*                      */
                      /*          I           */
                      /*                      */
                      /*     DECLARATIONS     */
                      /*                      */
                      /************************/
                    /*::::::::::::::::::::::::::*/

  CODE=4000
  
  CONSTANTS
  color = true
/* nocolor = true */
  del = 50

  DOMAINS
  
  list= string*   
  listlist=list*
  move= move(list)
  file=dialog


  DATABASE
  
                      /************************/
                      /*                      */
                      /*    BLOCKS WORLD      */
                      /*                      */
                      /************************/

  station(string,string)
  species(string,string,integer)
  station1(integer,string,string)
  oldcursor(string,integer,integer) 
  


  
                      /************************/
                      /*                      */
                      /*       LEXICON        */
                      /*                      */
                      /************************/
                                
  lexeme(string,string,list)




                      /************************/
                      /*                      */
                      /*      BLACKBOARD      */
                      /*                      */
                      /************************/

  modality(string)
  modality1(integer,string)
  np_anaphor(list)
  np_anaphor1(integer,list)
  anaphor_stack(listlist)
  n_anaphor(string)
  focus_prep(string)
  number(integer)
  dialog(string)
  continuation(string)
  continuation1(integer,string)
  order(integer,list)
  world_number(integer)
  
  CLAUSES
         
  /*----------------------------Blackboard-------------------------*/
  
    modality("!").
    focus_prep("").
    continuation(""). 
    order(0,[]).
    np_anaphor([]).
    n_anaphor("").
    anaphor_stack([]).
    number(0).
    world_number(0).
  /*---------------------------------------------------------------*/
  

                         /************************/
                         /*                      */
                         /*    INCLUDEFILES      */
                         /*                      */
                         /************************/


  include "control.blk"               /* CONTROL PREDICATES */
  include "world.blk"                 /* BLOCKSWORLD */
  include "io.blk"                    /* INPUT/OUTPUT */
  ifdef color include "color.blk"     /* DISPLAY */
  elsedef include "nocolor.blk" 
  enddef




                    /*::::::::::::::::::::::::::*/
                      /************************/
                      /*                      */
                      /*          II          */
                      /*                      */
                      /*      INTERPRETER     */
                      /*                      */
                      /************************/
                    /*::::::::::::::::::::::::::*/


   /*---------------------Interpreter Predicates------------------*/

  PREDICATES
   
  EXECUTE(list)
  VERIFY(string,list,string,string)
  IMAGINE(list)

  call(list)
  identify(list,list)
  find(string,string)
  new(string,string)

  /*----------------------Semantic operations----------------------*/
  
  comprehend(list,string)
  anaphorize(string,list)  
  n_anaphorize(list)

  /*----------------------Blocks operations------------------------*/

  perform(list,string,string)
  

  CLAUSES


                         /************************/
                         /*                      */
                         /*       COMMANDS       */
                         /*                      */
                         /************************/

  /*-----------------------------Execution-------------------------*/

  EXECUTE([Relation|Args]):- 
    anaphorize(user,Args),
    call([Relation|Args]),!.
  
  EXECUTE([Relation|Args]):- 
    identify(Args,Args1),
    not(Args=Args1),
    IMAGINE([Relation|Args1]),
    call([Relation|Args1]),!,
    anaphorize(user,Args1).

  /*-----------------------------Calls----------------------------*/


  call([elevate,X]):-
      elevate(X),!.
  call([clear,X]):- 
      clear(X),!.
  call([move,X,Y]):-
      move(X,Y),!.
  call([stack,X]):-
      stack(X),!.
  call([remove,Topos]):-
      remove(Topos),!.
  call([regard,Topos]):-
      random(X),
      X>0.3,
      species(Topos,_,_),
      continuation(""),
      transform("",Topos,Answer),
      concat("OK - Blockhead is looking at ",Answer,String),
      in_dialog(String),
      write(String,"\n"),!;
      species(Topos,_,_),!.

  /*-------------------------Blocks-identification-----------------*/
  
  identify([],[]).
  
  identify([Block1|Blocklist],[Outblock1|Outblocklist]):-
      identify(Blocklist,Outblocklist),
      find(Block1,Outblock1).
  
  /*-----------------------------Find'em---------------------------*/

  find(Block,Outblock):-
 
  /* Specific block */
    
      species(Block,_,_),!,
      Outblock=Block;

  /* Unspecified block */
  
      fronttoken(Block,"IDENTIFY",Clause),
      MAKE_LIST(Clause,Clause1),
      VERIFY(blockhead,Clause1,Outblock,_),  
      Outblock<>"0",
      new(Block,Outblock);

      ISA(Type,Block),
      species(Outblock,Type,_),
      new(Block,Outblock);

   /* Under-specified block */
      
      Block=ablackblock,!,
      species(Outblock,_,_),
      MEMBER(Outblock,[smallblackblock,bigblackblock]).


  /*---------------------------New------------------------------*/

   new(Block,Outblock):-
      modality("!"),
      fronttoken(Block,Token,_),
      member(Token,[ablock,"IDENTIFY"]),
      world_number(X),X>0,modality("!"),
      !,np_anaphor(Anaphors),
      not(member(Outblock,Anaphors)).
   new(_,_):-!.






                     /************************/
                     /*                      */
                     /*        QUERIES       */
                     /*                      */
                     /************************/


  PREDICATES
  
  check(list)
  find_answer(string,list,list,string,string)
  several(string,list,string,string)
  
  CLAUSES  

  /*--------------------------Verification--------------------------*/


  VERIFY(V,[Relation|[Arg1|Arg2]],Arg1,"yes"):-
      anaphorize(V,[Arg1|Arg2]),
      check([Relation|[Arg1|Arg2]]).

  VERIFY(V,[Relation|Args],Focus,Answer):-
      identify(Args,Args1),
      not(Args=Args1),
      check([Relation|Args1]),
      find_answer(V,[Relation|Args],Args1,Focus,Answer).
      
  VERIFY(_,[_|[Arg1|Arg2]],"0",Answer):-

      species(Arg1,_,_),Arg2=[A2],
      not(species(A2,_,_)),
      concat("no ",A2,B2),
      transform(B2,"",Answer),!;
      
      not(species(Arg1,_,_)),
      concat("no ",Arg1,B1),
      transform(B1,"",Answer),!;

      Answer="no ",!.
      

  /*-----------------------------Relations--------------------------*/

  check(["answer",Top]):-
      species(Top,_,_),!.      


  check(["stat",Top,Loc]):-
      station(Top,Loc).

  check(["statinvers",Loc,Top]):-
      station(Top,Loc).

  check(["contain",Loc,Top]):-
      station(Top,Loc),
      Loc=box,!;

      bound(Loc),bound(Top),
      not(Loc=box),
      message("Only the box can contain something"),
      exit(1).

  check(["est",Top,Top]):-bound(Top),!.

  
  
  /*--------------------------Answer-generation-------------------*/
  
  find_answer(V,[Relation|[A1|A2]],[B1|B2],Focus,Answer):-

      /* Both arguments variables of same type */

     not(A1=B1), not(A2=B2), A2=[C2], B2=[D2],
     frontstr(1,A1,X,_),concat(X,_,C2),
     transform(A1,B1,C1),
     asserta(focus_prep("on")),
     transform("",D2,E2),
     retract(focus_prep("on")),
     concat(C1,E2,Answer),
     Focus=B1,
     anaphorize(V,[B1|B2]);

      /* Second argument of xa-type */

      A2=[C2],concat(x,_,C2),!,
      B2=[D2],
      transform(C2,D2,Answer),
      Focus=D2,
      anaphorize(V,[B1|B2]);

      A1=B1, A2=[C2],
      B2=[D2],
      transform(C2,D2,Answer),
      Focus=D2,
      anaphorize(V,[B1|B2]);
 
     /* First argument of xa-type */
     
     several(V,[Relation|[A1|A2]],B1,Answer),
     Focus=B1.



   several(V,[Relation|[Arg1|Arg2]],X,Answer):-
      check([Relation|Args]),
      Args=[X1|Arg2], X<>X1,
      ISA(Arg1,X1),
      Answer=several,!;
 
      anaphorize(V,[X|Arg2]),
      transform(Arg1,X,Answer),!.



                     /************************/
                     /*                      */
                     /*     IMAGINATION      */
                     /*                      */
                     /************************/

  PREDICATES
  
  imagination(list)
  do

  /*-----------------------Possible worlds----------------------*/

  push_world(integer)
  pop_world(integer)
  kill_worlds
  change_world_number(integer,integer)


  CLAUSES
  
  /*-----------------------Skip imagination----------------------*/

  IMAGINE(Proposition):-
      continuation(""),
      world_number(0),!;
 
  /*------------------------Use imagination----------------------*/
  
      imagination(Proposition).

  /*-------------------------Imagination terminated-----------------*/
 
  imagination(Proposition):- 
      continuation(""),!,
      modality(M),
      perform(Proposition,M,Value),
      Value="1",
      anaphorize(finished,[]),
      np_anaphor(Ana),
      world_number(Number),
      assertz(order(Number,Proposition)), 
      kill_worlds,
      do,
      retract(np_anaphor(_)),
      assertz(np_anaphor(Ana)),
      exit(1).
  
  /*--------------------Imagine action, change world---------------*/

  imagination(Proposition):-
      not(continuation("")),
      modality(M),
      world_number(Number), 
      push_world(Number),
      change_world_number(+1,_),
      perform(Proposition,M,Value),
      Value="1",
      assertz(order(Number,Proposition)), 
      anaphorize(finished,[]),
      comprehend(Next,_),
      imagination(Next),!.




  /*-------------------------Unimaginable--------------------------*/

  imagination(_):- 
      change_world_number(-1,Number),
      pop_world(Number),
      retract(order(Number,_)),
      fail.



  /*-------------------------Execute imagination-------------------*/

  do:-
      retract(order(_,Order)),
      EXECUTE(Order),
      fail;
      assert(order(0,[])).



  /*---------------------------Possible worlds----------------------*/
  
  push_world(Number):-
      modality(X),
      assert(modality1(Number,X)),
      continuation(Conj),
      assert(continuation1(Number,Conj)),
      np_anaphor(Anaph),
      assert(np_anaphor1(Number,Anaph)),fail;

      station(Z,W),assert(station1(Number,Z,W)),fail;
    
      !.
    
  pop_world(Number):-
      retract(modality(_)),
      retract(continuation(_)),
      anaphorize(finished,[]),
      np_anaphor(Anaph),
      retract(np_anaphor(Anaph)),
      retract(station(Z,W)),fail;
  
      retract(modality1(Number,X)),
      assert(modality(X)),
      retract(continuation1(Number,Conj)),
      assert(continuation(Conj)),
      retract(np_anaphor1(Number,Anaph)),
      assert(np_anaphor(Anaph)),
      fail;
  
      retract(station1(Number,U,V)),
      assert(station(U,V)),fail;
      !.

  kill_worlds:-
      not(world_number(0)),
      repeat,
        change_world_number(-1,_),
        world_number(Number),
        pop_world(Number),
        Number=0,!;
      !.
              
  change_world_number(PlusMinusOne,Number):-
      world_number(Number1),
      Number=Number1+PlusminusOne,
      retract(world_number(Number1)),
      assert(world_number(Number)),!.



















                   /*::::::::::::::::::::::::::*/
                     /************************/
                     /*                      */
                     /*          III         */
                     /*                      */
                     /*        GRAMMAR       */
                     /*                      */
                     /************************/
                   /*::::::::::::::::::::::::::*/


                     /************************/
                     /*                      */
                     /*       LEXICON        */
                     /*                      */
                     /************************/

  PREDICATES

  syntaxeme(string,string,string) 

  include "lexicon.blk"



                     /************************/
                     /*                      */
                     /*       SEMANTICS      */
                     /*                      */
                     /************************/


  /*-----------------------Anaphorization-------------------------*/
  
  PREDICATES
  
  test_anaphor(list,list)
  test_anaphor_stack(list)

  CLAUSES
  
  anaphorize(blockhead,_):-!.

  anaphorize(user,NP):-
      test_anaphor(NP,NP1),
      test_anaphor_stack(NP1),!,
      n_anaphorize(NP1).  
  
  anaphorize(finished,[]):-
      retract(anaphor_stack([Base|_])),
      retract(np_anaphor(_)),
      assert(np_anaphor(Base)),
      assert(anaphor_stack([])),!.

  anaphorize(_,_).

  test_anaphor([],[]):-!.
  test_anaphor([NP|Tail],[NP|Tail1]):-
    species(NP,_,_),!,
    test_anaphor(Tail,Tail1).
  test_anaphor([_|Tail],Tail1):-!,
    test_anaphor(Tail,Tail1).
  
  test_anaphor_stack(NP):-
      anaphor_stack([NP|_]),!;
      retract(anaphor_stack(T)),!,
      assert(anaphor_stack([NP|T])).
      
  /*------------------anaphorize single Noun------------------------*/
  
  n_anaphorize([Noun]):-
      species(Noun,Nounclass,_),
      retract(n_anaphor(_)),!,
      asserta(n_anaphor(Nounclass));
      
      ISA(Nounclass,Noun),
      retract(n_anaphor(_)),!,
      asserta(n_anaphor(Nounclass)).

  n_anaphorize([_|Nounlist]):-
      !,n_anaphorize(Nounlist). 

  n_anaphorize(_):-!.





                     /************************/
                     /*                      */
                     /*        SYNTAX        */
                     /*                      */
                     /************************/


  /*--------------------------Syntactic rules----------------------*/
  
  PREDICATES

  S(list,list)
  focus(list,list,list)
  predicate(list,list,list)
  argument_field(list,move,list,list,list)
  Aux_Verb(list,list,list)
  PrepP(list,list,string)
  NP(list,list,string)
  unique(list,list,string)
  non_unique(list,list,string)
  restrictor(list,list,string,string)
  adjunct(list,list,string)
  identifier(list,list,string,string)
  Nom(list,list,string)
  ADJP(list,list,string)
  RelCl(list,list,list,move)
  PrepMod(list,list,string,string)
  exform(list,list,string,list)    


  /*-------------------------Sentence------------------------------*/

  CLAUSES

  /* With "focus" */

  S([Voc|Entry],Sem):-
       not(lexeme(Voc,v,_)),
       focus ([Voc|Entry],Difference1,Sem1),
       predicate (Difference1,Difference2,[Rel|Roles]),
       n_anaphorize(Sem1),
       argument_field(Roles,move(Sem1),Difference2,Remainder,Sem2),
       Remainder=[],!,
       APPEND([Rel],Sem2,Sem).


  /* Without "Focus" */
  
  S(Entry,Sem):-
       predicate(Entry,Difference1,[Rel|Roles]),
       argument_field (Roles,move([]),Difference1,Remainder,Sem1),
       Remainder=[],!,
       APPEND([Rel],Sem1,Sem).
  
  /* Error */
  
  S(_,Sem):-Sem=[].   
 
  /*-------------------------Focus---------------------------------*/
      
  /* Prepositional focus */
  
  focus([Voc|Entry],Remainder,[Sem]):-
       PrepP([Voc|Entry],Remainder,Sem),
       asserta(focus_prep(Voc)),!.

  /* Double focus */

  focus(Entry,Remainder,Sem):-
       NP(Entry,Difference1,Sem1),
       Difference1=["does"|Difference2],
       NP(Difference2,Remainder,Sem2),
       Sem=[Sem2,Sem1],
       asserta(focus_prep("")),!.
  
  /* Nominal focus */

  focus(Entry,Remainder,[Sem]):-
       NP(Entry,Remainder,Sem),
/*       Remainder=[Voc|_],
       lexeme(Voc,v,_),
*/       asserta(focus_prep("")).
  
  /*-------------------------Predicate-----------------------------*/

  /* Auxiliary + Verb */
  
  predicate(Entry,Remainder,Sem):-
     Aux_Verb(Entry,Remainder,Sem).
 
  /* Imperative and "is" */

  predicate([Verb|Remainder],Remainder,Sem):-
       lexeme(Verb,v,Sem).

  /* Present tense */
  
  predicate([Voc|Entry],Entry,Sem):-
       exform([Voc],Sem,"s",_),!.

  /*----------------------Auxiliary verb---------------------------*/

                          /* "does" + Infinitive */
  
  Aux_Verb(["does",Infinitive|Remainder],Remainder,Sem):-
       lexeme(Infinitive,v,Sem),!.
       
  Aux_Verb(["does","there",Infinitive|Remainder],Remainder,Sem):-
       lexeme(Infinitive,v,Sem),!.
  
  Aux_Verb(["does"|Entry],Remainder,Infinitive):-   
       exform(Entry,Infinitive,"",Remainder),!.



                       /* "is there"/ " there is" */
  
  Aux_Verb(["is","there"|Entry],Remainder,Sem):-
       predicate(["is"|Entry],Remainder,Sem),!.

  Aux_Verb(["there","is"|Entry],Remainder,Sem):-
       predicate(["is"|Entry],Remainder,Sem),!.
  

                       /* "is ... -ing" */
  
  Aux_Verb(["is"|Entry],Remainder,Sem):-
       exform(Entry,Sem,"ing",Remainder),!;

                       /* "is ... -ed" */
  
       exform(Entry,Sem,"ed",Remainder),!.


  /*-------------------------Arguments-----------------------------*/
  
                            /* Agent */
  
  argument_field([agt|Roles],move(X),Entry,Remainder,Sem):-
       argument_field(Roles,move(X),Entry,Remainder,Sem).

                        /* Topos,Locus */
  
  argument_field([top,loc],move([Sem1|X]),Entry,Remainder,
     [Sem1|Sem2]):-
       n_anaphorize([Sem1]),
       argument_field([loc],move(X),Entry,Remainder,Sem2).
 
  argument_field([top,loc],move(X),Entry,Remainder,Sem):-
       Entry=[down|Difference],
       NP(Difference,Remainder,Sem1),
       n_anaphorize([Sem1]),
       Sem=[Sem1,table];
   
       NP(Entry,Difference,Sem1),
       n_anaphorize([Sem1]),
       argument_field([loc],move(X),Difference,Remainder,Sem2),
       APPEND([Sem1],Sem2,Sem).

               /* Locus = "from" <source> "to" <destination> */

  argument_field([top,loc],move([Sem1|X]),[from|Entry],
       Remainder,[Sem1|Sem2]):-
       n_anaphorize([Sem1]),
       NP(Entry,Difference,Sem3),
       VERIFY(blockhead,[stat,Sem1,Sem3],TruthValue,_),
       TruthValue<>"0", 
       argument_field([loc],move(X),Difference,Remainder,Sem2).
   
  argument_field([top,loc],move(X),Entry,Remainder,Sem):-
       Entry=[down|Difference],
       NP(Difference,Difference1,Sem1),
       Difference1=[from|Difference2],
       NP(Difference2,Remainder,Sem3),
       VERIFY(blockhead,[stat,Sem1,Sem3],TruthValue,_),
       TruthValue<>"0", 
       n_anaphorize([Sem1]),
       Sem=[Sem1,table];

       NP(Entry,Difference1,Sem1),
       n_anaphorize([Sem1]),
       Difference1=[from|Difference2],
       NP(Difference2,Difference3,Sem3),
       VERIFY(blockhead,[stat,Sem1,Sem3],TruthValue,_),
       TruthValue<>"0", 
       argument_field([loc],move(X),Difference3,Remainder,Sem2),
       APPEND([Sem1],Sem2,Sem).

                           /* Locus,Topos */

  argument_field([loc,top],move([Sem1|X]),Entry,Remainder,Sem):-
       n_anaphorize([Sem1]),
       argument_field([top],move(X),Entry,Remainder,Sem2),
       APPEND([Sem1],Sem2,Sem).
  
  argument_field([loc,top],move([]),Entry,Remainder,Sem):-
       NP(Entry,Difference,Sem1),
       n_anaphorize([Sem1]),
       argument_field([top],move([]),Difference,Remainder,Sem2),
       APPEND([Sem1],Sem2,Sem).
  
                            /* Topos,Topos */
  
  argument_field([top,top],move([Sem1|X]),Entry,Remainder,Sem):-
       n_anaphorize([Sem1]),
       argument_field([top],move(X),Entry,Remainder,Sem2),
       APPEND([Sem1],Sem2,Sem).
  
  argument_field([top,top],move(X),Entry,Remainder,Sem):-
       NP(Entry,Difference,Sem1),
       n_anaphorize([Sem1]),
       argument_field([top],move(X),Difference,Remainder,Sem2),
       APPEND([Sem1],Sem2,Sem).
    
                             /* Topos */
  
  argument_field([top],move([]),Entry,Remainder,[Sem]):-
       Entry=[up|Difference],!,
       NP(Difference,Remainder,Sem),
       n_anaphorize([Sem]);

       NP(Entry,Remainder,Sem),
       n_anaphorize([Sem]);
   
       NP(Entry,Difference,Sem),
       Difference=[up],!,
       Remainder=[],
       n_anaphorize([Sem]).
   
  argument_field([top],move(Sem),Entry,Entry,Sem):- 
       not(Sem=[]).
  
                             /* Locus */

  argument_field([loc],move([]),Entry,Remainder,[Sem]):-
       PrepP(Entry,Remainder,Sem).

  argument_field([loc],move(Sem),Entry,Entry,Sem):- 
       not(Sem=[]),focus_prep(P),not(P="").

  argument_field([loc],move(Sem),[Voc|Remainder],Remainder,Sem):-
       syntaxeme(Voc,"Prep",_),not(Sem=[]),focus_prep("").

  argument_field([loc],move([]),[Voc|Remainder],Remainder,[Sem]):-
       syntaxeme(Voc,"PrepP",Sem),!.

  argument_field([loc],move([]),[Part],Remainder,[table]):-
       MEMBER(Part,[up,down]),
       Remainder=[],!.


  /*-----------------Prepositional Phrases-----------------------*/
  
  PrepP(["on","top","of"|Entry],Remainder,Sem):-
       NP(Entry,Remainder,Sem),!.
  
  PrepP(["where"|Remainder],Remainder,Sem):-
       Sem=x,!.
  
  PrepP([Voc,Voc1|Entry],Remainder,Sem):-
       syntaxeme(Voc,"Part",_),
       syntaxeme(Voc1,"Prep",_),
       NP(Entry,Remainder,Sem),!.

  PrepP(["on","top"],[],Sem):-
       syntaxeme("top","PrepP",Sem),!.
   
  PrepP([Voc|Entry],Remainder,Sem):-
       syntaxeme(Voc,"Prep",_),
       NP(Entry,Remainder,Sem).


  /*-------------------------Noun Phrases------------------------*/
  
  /* (1) definite NP with unique reference*/
  
  NP([Voc|Entry],Remainder,Sem):-

       syntaxeme(Voc,"Def",_),
       unique(Entry,Difference,Sem),
       adjunct(Difference,Remainder,Sem).

  /* (2) Indefinite NPs */
  
  NP([Voc|Entry],Remainder,Sem):-

       syntaxeme(Voc,"Indef",X),
       non_unique([X|Entry],Difference,Sem1),
       identifier(Difference,Remainder,Sem1,Sem).
  
  /* (3) Pronominal NP */    

  NP([Voc|Entry],Remainder,Sem):-

       syntaxeme(Voc,"Pron",Sem1),
       identifier(Entry,Remainder,Sem1,Sem).

  /*----------Nominal Expressions with unique reference------------*/

     unique([Voc|Entry],Remainder,Sem):-
       
  /* (1) With Adjective phrase */
  
       lexeme(Voc,Class,_),member(Class,[adj,adj1,adj2]),
       ADJP([Voc|Entry],Difference,Sem1),
       Nom(Difference,Difference1,Sem2), 
       concat(Sem1,Sem2,Sem3),
       species(Sem3,_,_),
       restrictor(Difference1,Remainder,Sem3,Sem),!; 

  /* (2) Unique in Universe */      

       lexeme(Voc,n,[Sem1]), 
       findall(Entity,species(Entity,Sem1,_),UnitClass),
       UnitClass=[Sem],Remainder=Entry,!;

  /* (3) Unique by Restrictor */
  
       Nom([Voc|Entry],Entry,Sem1), 
       restrictor(Entry,Remainder,Sem1,Sem);

  /* (2b) Unique in Discourse (Anaphor) */

       Nom([Voc|Entry],Entry,Sem1), 
       np_anaphor(List),
       species(Sem,Sem1,_),
       member(Sem,List),Remainder=Entry;
       
  /* (4) Unique by Prepositional Modifier and Restrictive Relative */

       Nom([Voc|Entry],Entry,Sem1), 
       PrepMod(Entry,Difference,Relation,Sem2),
       restrictor(Difference,Remainder,Sem1,Sem),
       VERIFY(blockhead,[Relation,Sem,Sem2],TruthValue,_),
       TruthValue<>"0",!.
            

  /*----------Nominal Expressions without reference-------------*/

    non_unique([Voc1,Voc2|Entry],Remainder,Sem):-

  /* (1) Singleton hence Unique*/ 
  
       lexeme(Voc2,Class,_),member(Class,[adj,adj1]),
       unique([Voc2|Entry],Remainder,Sem),!;
  
  /* (2) With Adjective phrase, not "which" */
       
       Voc1<>"x",
       lexeme(Voc2,Class,_),member(Class,[adj1,adj2]),
       ADJP([Voc2|Entry],Difference,Sem1),
       Nom(Difference,Difference1,Sem2), 
       concat(Sem1,Sem2,Sem3),
       concat("a",Sem3,Sem4),
       identifier(Difference1,Remainder,Sem4,Sem),!;
  
  /* (3) Singleton, not "which" */
       
       Voc1<>"x",
       lexeme(Voc2,Object,_),species(Object,_,_),
       unique([Voc2|Entry],Remainder,Sem),!;

  /* (4) Simple with Identifier */
       
       Nom([Voc2|Entry],Entry,Sem1),
       concat(Voc1,Sem1,Sem2), 
       identifier(Entry,Remainder,Sem2,Sem),!.
 

  /*-------------------------Nominal-------------------------------*/

  /* (1) Noun */
    
    Nom([Voc|Remainder],Remainder,Sem):-
      lexeme(Voc,n,[Sem]).

  /* (2) "one" */ 
  
    Nom([one|Remainder],Remainder,Sem):-
      syntaxeme(one,"N-substitute",Sem).



  /*-------------------------Adjunct-------------------------------*/

    adjunct(Entry,Remainder,Sem):-  

  /* (1) Parenthetic relative clause */

       Entry= [X,Y|_],
       member("which",[X,Y]), 
       RelCl(Entry,Remainder,Return,move([Sem])),
       VERIFY(blockhead,Return,TruthValue,_),
       TruthValue<>"0",!; 

  /* (2) Empty */
       
       Remainder=Entry;

  /* (3) Prepositional phrase */
       
       PrepMod(Entry,Remainder,Relation,Sem1),
       VERIFY(blockhead,[Relation,Sem,Sem1],TruthValue,_),
       TruthValue<>"0". 


  /*-------------------------Restrictor---------------------------*/

    restrictor(Entry,Remainder,Sem1,Sem):-
      
  /* (1) Empty */

      species(Sem1,_,_),Sem=Sem1,!,Entry=Remainder;
  
  /* (2) Restrictive relative clause */
      
      concat("x",Sem1,Sem2),
/*      find(Sem2,Sem3),*/
      RelCl(Entry,Remainder,Return,move([Sem2])),
      VERIFY(blockhead,Return,TruthValue,X),
      TruthValue<>"0",
      X<>"several",
      TruthValue=Sem;

  /* (3) Prepositional Modifier */
      
      PrepMod(Entry,Remainder,Relation,Sem2),
      concat("a",Sem1,Sem3),
      VERIFY(blockhead,[Relation,Sem3,Sem2],Sem,X),
      Sem<>"0",
      X<>"several".
      

  /*-------------------------Identifiers---------------------------*/
  
    identifier(Entry,Remainder,Sem1,Sem):-

  /* (1) Relative clause */
       
       frontstr(1,Sem1,X,_),X="a",
       RelCl(Entry,Remainder,Return,move([Sem1])),
       MAKE_STRING(["IDENTIFY"|Return],Sem),!;

  /* (2) Empty */
       
       frontstr(1,Sem1,X,_),
       member(X,[a,x]),
       Entry=Remainder,
       Sem=Sem1;

       species(Sem1,_,_),
       Entry=Remainder,
       Sem=Sem1,!;
       
  /* (3) Prepositional modifier */
  
       PrepMod(Entry,Remainder,Relation,Sem2),
       MAKE_STRING(["IDENTIFY",Relation,Sem1,Sem2],Sem);
         
  /* (4) Catch all */
    
       Entry=Remainder,Sem=Sem1,!.


  /*-------------------------Relative Clauses----------------------*/


  /* (1) With Relative Subject */

  RelCl([Voc|Entry],Remainder,Sem,move(Sem1)):-
       syntaxeme(Voc,"Subject-relative",_),
       predicate(Entry,Difference,[Rel|Roles]),
       n_anaphorize(Sem1),
       argument_field(Roles,move(Sem1),Difference,Remainder,Sem2),
       APPEND([Rel],Sem2,Sem);
      
   /* (2) Oblique Relative pronoun with "stranded" preposition */
 
       syntaxeme(Voc,"Oblique-relative",_),
       NP(Entry,Difference,Sem2),
       predicate(Difference,Difference1,[Rel|Roles]),
       argument_field(Roles,move([Sem2|Sem1]),Difference1,
           Remainder,Sem3),
       APPEND([Rel],Sem3,Sem);
    
  /* (3) Omitted relative pronoun */

       not(member(Voc,["which","that"])),
       NP([Voc|Entry],Difference,Sem2),
       not(Difference=[]),
       predicate(Difference,Difference1,[Rel|Roles]),
       n_anaphorize([Sem2]),
       argument_field(Roles,move([Sem2|Sem1]),Difference1,
           Remainder,Sem3),
       APPEND([Rel],Sem3,Sem).
    
  /* (4) Oblique Relative pronoun with prefixed preposition */  
  
  RelCl([Voc,Voc1|Entry],Remainder,Sem,move(Sem1)):-
       syntaxeme(Voc,"Prep",_),
       syntaxeme(Voc1,"Oblique-relative",Mark),
       Mark="+prep",
       NP(Entry,Difference,Sem2),
       predicate(Difference,Difference1,[Rel|Roles]),
       assert(focus_prep(Voc)),
       argument_field(Roles,move([Sem2|Sem1]),Difference1,
          Remainder,Sem3),
       retract(focus_prep(Voc)),
       APPEND([Rel],Sem3,Sem);
  
 /*(5) Relative-Oblique with prefixed preposition and formal subject*/
       syntaxeme(Voc,"Prep",_),
       syntaxeme(Voc1,"Oblique-relative",Mark),
       Mark="+prep",
       predicate(Entry,Difference,[Rel|Roles]),
       NP(Difference,Difference1,Sem2),
       assert(focus_prep(Voc)),
       argument_field(Roles,move([Sem2|Sem1]),Difference1,
          Remainder,Sem3),
       retract(focus_prep(Voc)), 
       APPEND([Rel],Sem3,Sem).

  /* (6) Reduced Relative Clause */
  
  RelCl([Voc|Entry],Remainder,Sem,move(Sem1)):-
       lexeme(Voc1,v,_),concat(Voc1,X,Voc),X<>"s",
       predicate(["is",Voc|Entry],Difference,[Rel|Roles]),!,
       argument_field(Roles,move(Sem1),Difference,Remainder,Sem2),
       APPEND([Rel],Sem2,Sem).


  /*---------------------Prepositional Modifier--------------------*/
  
   PrepMod([Voc|Entry],Remainder,Relation,Sem):-
       syntaxeme(Voc,"Prep",Relation),
       not(Relation="dyn"),
       NP(Entry,Remainder,Sem).
       

  /*-------------------------Adjective Phrase----------------------*/

  ADJP([Voc|Remainder],Remainder,Sem):-
      lexeme(Voc,adj,Sem1),
      Sem1=[Sem],!.
 
  ADJP([Voc,Voc1|Remainder],Remainder,Sem):-
      lexeme(Voc,adj1,[Sem1]),
      lexeme(Voc1,adj,[Sem2]),!,
      concat(Sem1,Sem2,Sem);
    
      lexeme(Voc,adj2,[Sem1]),
      lexeme(Voc1,adj,[Sem2]),!,
      concat(Sem1,Sem2,Sem).
     


  /*-----------Morphological analysis of verbal forms--------------*/
    
  exform([Form|T],Sem,Morph,T):-
      concat(Voc,Morph,Form),
      Morph="ed",
      lexeme(Form,v,Sem),!;
    
      concat(Voc,Morph,Form),
      lexeme(Voc,v,Sem),!.
 
  exform([Voc|T],Sem,Morph,[Voc|T1]):-
      exform(T,Sem,Morph,T1).  

   















                   /*::::::::::::::::::::::::::*/
                     /************************/
                     /*                      */
                     /*          IV          */
                     /*                      */
                     /*      SUPERVISOR      */
                     /*                      */
                     /************************/
                   /*::::::::::::::::::::::::::*/



  PREDICATES

  supervisor
  exit_mode(integer)
    
  /*----------------------------Start---------------------------*/

  GOAL 
    makewindow(2,7,7," BLOCKHEAD ",18,0,7,80),
    shiftwindow(2),
    makewindow(1,112,7," BLOCKHEAD ",0,0,18,80),
    shiftwindow(1),
    assert(dialog("")),
    nl,
    nl,write("                             B L O C K H E A D  "),
    nl,write("                                     A  "),
    nl,write("                         natural language program"),
    nl,write("                              in TURBO-PROLOG"),
    nl,write("                                 commanding  "),
    nl,write("                                  a robot  "),
    nl,write("                             @ Henrik Prebensen,  "),
    nl,write("                           Centre for Informatics "),
    nl,write("                             Faculty of Letters "),
    nl,write("                          University of Copenhagen  "),
    nl,write("                             Version 28.01.88 "),
    nl,
    D=6*del,randomize(20),
    DELAY(D),stopcolor,clearwindow,
    show(world),

  /*-------------------------Trap loop---------------------------*/
  
    repeat,
      trap(supervisor,Exit,exit_mode(Exit)),
    fail.


  CLAUSES
  
  exit_mode(0):- kill_worlds,assert(continuation("")),!,exit(0).
  exit_mode(1):- kill_worlds,assert(continuation("")),!.
 
  /*----------------------Supervisor loop------------------------*/
  
  supervisor:-    
    tabula_rasa,
    shiftwindow(2),
    repeat,
      randomize(10),
      comprehend(Meaning,Modality),
      perform(Meaning,Modality,Value),
      message(Value),
      tabula_rasa,
    fail.
  
  /*-------------------------Comprehension-------------------------*/
    
  comprehend(Meaning,Modality):-
      input(SimpleSentence),
      S(SimpleSentence,Meaning),
      modality(Modality),!;
  
      Meaning=[],modality(Modality),!.


  /*--------------------------Performance--------------------------*/
  
  /* (1) Error */
  
  perform([],_,"1"):-!,
      message("The sentence is deficient"),
      exit(1).
  
  /* (2) Query */
          
  perform(Question,Modality,"1"):-
      MEMBER(Modality,["?","."]),
      VERIFY(user,Question,_,Answer),
      answer(Answer),!.

  /* (3) Command */
  
  perform(Order,"!","1"):-
     
     /* simple order */
  
      continuation(""),
      EXECUTE(Order),!;
     
     /* complex order in real world */

      not(continuation("")),
      retract(order(0,[])),
      IMAGINE(Order),!;

     /* complex order in possible world */
  
      not(continuation("")),
      not(world_number(0)),
      EXECUTE(Order),!;
  
     /* complex order inconsistent with world */

      not(continuation("")),
      world_number(0),!,  
      message("Sorry, the order is inexecutable"),!,
      exit(1).

     /* general failure */
  
  perform(_,"!","Sorry, the order seems inexecutable").

  /* (4) Condition */  

  perform(Condition,">","1"):-
      VERIFY(user,Condition,Truth_Value,_),
      Truth_Value<>"0",!;

      message("Sorry, the condition is false"),!,
      exit(1).

  /*----------------------Garbage collection-----------------------*/
  
  tabula_rasa:- 
      not(continuation("")),!,
      anaphorize(finished,[]).

  tabula_rasa:- 
      retract(continuation(_)),fail;
      assert(continuation("")),
      retract(modality(_)), fail;
      assert(modality("!")),
      retract(focus_prep(_)),fail;
      assert(focus_prep("")),
      retract(order(_,_)),fail;
      assert(order(0,[])),  
      anaphorize(finished,[]),fail;
      kill_worlds.
 
                         /***********************/
                         /*                     */
                         /*         END         */
                         /*                     */
                         /***********************/
                        
 
