/*
===========================================================================
	CHEMICAL.PRO   

	Main module for  CHEMICAL project

	Copyright (c) 1986, 1990 by Prolog Development Center
===========================================================================
*/

code=3999
project "chemical"

include "chemglob.pro"
include "chemutil.pro"



PREDICATES
  nondeterm repeat.  
  start. 
  go.
  make_sure(integer,integer,symbol,char).yes_no(char,char).
  read_name(symbol).print_name(symbol).
  make_decision(char). 
  process(symbol).
  process1(List,symbol).
  process2(symbol,Llist,Llist,Llist,Llist,Llist,Llist).
  nondeterm choices(integer,symbol).
  make_windows.
  error_handling.


GOAL
  go.
  
include "stems.dba"  
  

CLAUSES
  repeat.
  repeat:-repeat.
  yes_no('Y','y').yes_no('y','y').
  yes_no('N','n').yes_no('n','n'). 

  choices(1,"acetylene").
  choices(2,"chloroform").  
  choices(3,"carbontetrachloride").
  choices(4,"2,2-dimethylbutane").     
  choices(5,"2,2,4-trimethylpentane").
  choices(6,"2-methyl-3-ethylpentane").     
  choices(7,"2,4,4-trimethyl-2-pentene").     
  choices(8,"4-methyl-3,3-diethyl-5-isopropyloctane").
  choices(9,"2,2,3,3-tetramethylpentane").
  choices(10,"3,4-dimethyl-4-ethylheptane").
  choices(11,"2-methylpropene").
  choices(12,"3,3-dimethyl-1-butene").
  choices(13,"4-methyl-2-pentene").
  choices(14,"3,6-dimethyl-1-octene").
  choices(15,"3-chloropropene").
  choices(16,"1,1-dichloroethene").
  choices(17,"1-chloro-2-methyl-2-butene").
  choices(18,"3-methyl-4-ethyl-3-hexene").
  choices(19,"3,3,3-trifluoropropene").
  /*choices(20,"2-methyl-1-butene").
  choices(21,"1,2-dichloroethene").
  choices(22,"1-bromo-2-octene").
  choices(23,"1-butyne").
  choices(24,"2-butyne").
  choices(25,"4-methyl-2-pentyne").
  choices(26,"3,3-dimethyl-1-butyne").
  choices(27,"5-decyne").
  choices(28,"1,3-butadiene").
  choices(29,"1,4-pentadiene").
  choices(30,"2-methyl-1,3-butadiene").
  choices(31,"2,3-dimethyl-1,3-butadiene").
  choices(32,"1,2-propadiene").
  choices(33,"1,3,5-hexatriene").
  */ 

  make_windows:-
	makewindow(1,27,0,"",0,0,25,80),
	makewindow(2,7,26,"IUPAC_Name",18,10,7,60),
	makewindow(3,78,30,"IUPAC Name",20,10,5,60),
	makewindow(4,15,30,"Make_Decision",20,10,5,60),
	makewindow(6,15,15,"Error Messages",3,10,15,60),
	makewindow(7,27,27,"",0,70,4,10),
	makewindow(8,78,30,"Note",0,70,4,10),
	write(" R   is\n"),   
	write("  CH3"),          
	makewindow(5,15,15,"Instructions",0,0,25,80),
	write("\n\n  This program draws the"),
	write(" structures of ACYCLIC alkanes"),
	write(" alkenes,alkynes - with   or without "),
	write(" halogen,alkyl,alkenyl, and alkynyl radicals.\n\n"),
	write("Care should be taken in inputting names : \n\n"),
	write(" 1.Numerals have to be separated by a non-space"),
	write(" or a non-underscore character    - which may be a"),
	write(" comma,semicolon,colon,dash or slash.\n\n"),
	write(" 2.Numerals cannot be separated by a single dot.\n\n"),
	write(" 3.Numerals cannot be preceded by an alphabetic character;they must"),
	write("              be separated by a comma etc. as above.\n\n"),
	write(" 4.Insertion of un-necessary characters will cause syntax errors.\n"),
	write("   When splitting or hyphenation is necessary, as in 'sec_butyl',\n "),
	write("  use an underscore or just leave a space.\n"),
            /*  write(" 5.The use of round brackets is important in names like\n"),
	write("    4-(2-propenyl)-1-hepten-5-yne \n"),
	write("    DO NOT use any other character in its place.\n"),  
              */  write("\n\n Please press the space bar"),readchar(_),
	write("\n\n\n\n\n\n\n This program accepts the following non-systematic  \n"),
	write(" names retained by IUPAC:	               \n\n"),
	write("  i. unsubstituted alkanes with the prefix 'iso' or 'neo'\n"),
	write("  ii. ethylene  iii. allene  iv. isoprene  v.acetylene\n"),
	write("  vi. chloroform   vii. carbontetrachloride\n\n"),	     
	write("  Note the following system limitations:\n"),
	write("  - max. no. of C atoms in the main chain is 12\n"),
	write("  when alk(yl/enyl/ynyl) radical substitutes are present\n"),
	write("  otherwise it is 24; \n"),
	write("  - max. no. of primary C atoms in a radical is 4.\n\n\n").
                     








  make_sure(X,Y,Mess,YN):-
	repeat,
	cursor(X,Y),write(Mess),readchar(Yes),
	yes_no(Yes,YN),!.

  read_name(X):-
  	shiftwindow(2),clearwindow,
        write(" Type in the name (eg. 1,2-dichloro-4-methylheptane)"),
        nl,
        readln(X1),
        X1<>"",
        make_sure(3,2,"Correctly spelled (y/n)?",YN),YN='y',
        scanx(X1,XL),concat_el(XL,X2),
        upper_lower(X2,X2l),iupac_name(X2l,X),!.

  read_name(X):-
	findall(Name,choices(_,Name),LongList),
  	menu(5,10,LongList,Num),
                   choices(Num,X1),
                   scanx(X1,XL),
                   concat_el(XL,X2),
                   upper_lower(X2,X2l),iupac_name(X2l,X),!.

  print_name(X):-shiftwindow(3),clearwindow,
	upper_lower(Xu,X),
        str_len(Xu,L),
        L<59,!,Col=(58-L) div 2,
        cursor(1,Col),write(Xu),
        nl,write(" Press the space bar "),readchar(_).

  print_name(X):-
        shiftwindow(3),clearwindow,
        upper_lower(Xu,X),
        write(Xu),
        nl,write(" Press the space bar "),readchar(_).





  go:-
  	make_windows,
        write("\n\n Press the space bar to start"),
        readchar(_),
        start.



  start:- 
	assert(status(no_error)),
	repeat,
	  error_handling,
          shiftwindow(1),clearwindow,
          read_name(Nm),
          process(Nm),
          print_name(Nm),
          make_decision(Y),
          clearwindow,
	  Y='n',!,
	removewindow(1,1),
	removewindow(2,1),
	removewindow(3,1),
	removewindow(4,1),
	removewindow(5,1),
	removewindow(6,1),
	removewindow(7,1),
	removewindow(8,1).

  error_handling:-
        status(no_error),!.

  error_handling:-
 	shiftwindow(2),
        cursor(3,2),
        write(" Press the space bar     "),readchar(_),
        shiftwindow(6),clearwindow,
        assert(status(no_error)).

         
  process(X):-
 	retract(status(no_error)),
	scan(X,XL),filter(XL,XL2),filter_usc(XL2,XL3),
  	concat_prefix_alkyl(XL3,XL4),
  	syntax_check(XL4,Reply),
  	process1(XL4,Reply),!.

  process1(List,no_error):- 
  	semantic_check(List,L1,L2,L3,L4,L5,L6,Reply),
  	process2(Reply,L1,L2,L3,L4,L5,L6),!.

  process2(no_error,L1,L2,L3,L4,L5,L6):-
  	assert(status(no_error)),
  	shiftwindow(1),
  	draw(L1,L2,L3,L4,L5,L6,Flag),W=Flag+7,
  	shiftwindow(W),!.
  
  make_decision('n'):-
  	shiftwindow(4),
  	clearwindow,
  	make_sure(1,0," View another structure (y/n)?",YN),!,
  	YN='n',
  	cursor(1,26),write("BYE!").
  
  make_decision('y').  
         


       
     
     
     
     
                     
