/*
===========================================================================
	CHEMUTIL.PRO   

	utility predicates for CHEMICAL project

	Copyright (c) 1986, 1990 by Prolog Development Center
===========================================================================
*/


PREDICATES 
  member(integer,int_list,int_list)
  member(element,Llist,Llist) 
  count_N(integer,Int_list,integer)
  sort(int_list,int_list) 
  make_db_trb_list(Int_list,Int_list) 
  match(symbol,symbol,symbol) 
  sieve_int(List,symbol,List)
  scanx(symbol,list) 
  equal(symbol,symbol).equal(list,list).equal(int_list,int_list).
  check(symbol).check_list(list).check_list(int_list).
  check_list(Llist).check_int(symbol).    
  find(symbol,symbol,symbol,integer).
  left_str(integer,symbol,symbol).right_str(integer,symbol,symbol).
  nondeterm append(list,list,list). 
  nondeterm append(int_list,int_list,int_list). 
  nondeterm append(llist,llist,llist).
  list_len(List,integer). list_len(Int_list,integer).  
  make_list(integer,integer,Int_list).
  ourmax(int_list,integer).ourmin(int_list,integer).
  str_int_list(List,Int_list).
  delete(integer,int_list,int_list).delete(element,Llist,Llist).

   
   
CLAUSES
  equal(X,X).

  check_list([]). 
  
  check(""). /* To check whether the list/string Null */
     
  check_int(IntStr):-       /* To check whether the string integer */
	str_int(Intstr,_).                                                                


  sieve_int([Tok,Tok2|L],Ints,[Tok2|L]):-
        check_int(Tok),
        not(check_int(Tok2)),concat(Tok,":",Ints),!.
                                                                                
  sieve_int([Tok,Tok2|L],Ints3,L1):-
        check_int(Tok),sieve_int([Tok2|L],Ints2,L1),
        concat(Tok,":",Ints1),concat(Ints1,Ints2,Ints3),!.
            
  sieve_int([Tok],Ints,[]):-
        check_int(Tok),concat(Tok,":",Ints),!.

  sieve_int(L,":",L).    


  match(H,HT,T):-
        str_len(H,Lh),frontstr(Lh,HT,Fht,T),
        equal(H,Fht).
      
  find("",S,S,0):-!.     

  find(Tree,Forest,Rest,1):-
        match(Tree,Forest,Rest),!.

  find(Tree,Forest,Rest,N):-
        frontstr(1,Forest,_,Bforest),
        find(Tree,Bforest,Rest,N1),
        N=N1+1.
     
  left_str(N,LR,LR):-str_len(LR,L),L<=N,!.
  left_str(N,LR,L):-frontstr(N,LR,L,_).
     
  right_str(N,LR,LR):-str_len(LR,L),L<=N,!.
  right_str(N,LR,R):-str_len(LR,Lnr),
                     Llr=Lnr-N,frontstr(Llr,LR,_,R).
     
  append([],L,L).
  append([A1|L1],L2,[A1|L3]):-append(L1,L2,L3). 
    
  list_len([],0).
  list_len([_|List],N):- list_len(List,N1),N=N1 + 1.
    
  count_N(N,[N|List],M):-!,count_N(N,List,M1),M=M1+1.
  count_N(N,[_|List],M):-!,count_N(N,List,M).
  count_N(_,[],0).
    
  make_list(_,N,[]):-N<1,!.
  make_list(Int,N,[Int|List]):-N1=N-1, make_list(Int,N1,List).

  make_db_trb_list([],[]).
  make_db_trb_list([N|List],[N,N11|List1]):-
	N11=N+1,make_db_trb_list(List,List1).
    

  ourmax([M],M).
  ourmax([F,S|List],M):-
        F<S,!,  
        ourmax([S|List],M).
  ourmax([F,_|List],M):-ourmax([F|List],M).

  ourmin([M],M):-!.
  ourmin([F,S|List],M):-
	F<S,!, /*  dummy is needed (strange?!) */ 
  	ourmin([F|List],M).
  ourmin([_|List],M):-ourmin(List,M).

  sort(L,S):-
	append(X,[A,B|Y],L),B<A,
	append(X,[B,A|Y],M),sort(M,S),!.
  sort(L,L).

  str_int_list([],[]).
  str_int_list([Str|Slist],[Int|Ilist]):-
        str_int(Str,Int),str_int_list(Slist,Ilist).
                     
  member(X,[X|T],T):-!.
  member(X,[_|List],RList):-member(X,List,Rlist).
    
  delete(_,[],[]).
  delete(X,[X|T],L):-!,delete(X,T,L).
  delete(X,[Y|T],[Y|L]):-delete(X,T,L).
    



 /********************** Scanning the Name *********************************/
    
PREDICATES
  scan(symbol,list).
  concat_el(list,symbol).
  filter(list,list).
  filter_uscx(symbol,symbol).
  filter_usc(list,list).
  concat_prefix_alkyl(List,List).
  concat_prefix_alkylx(symbol,List,List).


CLAUSES
  scanx(Name,[Tok|L]):-
	fronttoken(Name,Tok,Rest),scanx(Rest,L),!.
  
  scanx(_,[]).
       
    
  concat_el([H1,H2|T],Nm):-
	concat(H1,H2,H),concat_el([H|T],Nm).
           
  concat_el([H],H).

  concat_prefix_alkylx(Px,[PxPfx,Alk|L],Lnew):-
	str_len(Px,Lpx),
	right_str(Lpx,PxPfx,Pfx),equal(Px,Pfx),
	concat(PxPfx,Alk,PxPfxAlk),
	concat_prefix_alkylx(Px,[PxPfxAlk|L],Lnew),!.
                  
  concat_prefix_alkylx(Px,[PxAlk|L],[PxAlk|Lnew]):-
        concat_prefix_alkylx(Px,L,Lnew),!.
 
  concat_prefix_alkylx(_,[],[]).
       
 
  concat_prefix_alkyl(L,Lnew):-
	concat_prefix_alkylx(iso,L,L1),
        concat_prefix_alkylx(neo,L1,L2),
        concat_prefix_alkylx(sec,L2,L3),
        concat_prefix_alkylx(tert,L3,Lnew).

  scan(Name,L):-
        scanx(Name,Lx),concat_el(Lx,Nmx),  /* To avoid in between spaces*/
        scanx(Nmx,L).
        
  filter([","|T],L):- filter(T,L),!.
  filter(["-"|T],L):- filter(T,L),!.
  filter(["_"|T],L):- filter(T,L),!.
  filter(["."|T],L):- filter(T,L),!.
  filter([";"|T],L):- filter(T,L),!.
  filter([":"|T],L):- filter(T,L),!.
  filter(["/"|T],L):- filter(T,L),!.
  filter([H|T],[H|L]):-filter(T,L),!.
  filter([],[]).
    
  filter_uscx(H,L):-
		frontchar(H,'_',Rest),
                filter_uscx(Rest,L),!.
    
  filter_uscx(H,H2):-
		frontchar(H,C,Rest),
                filter_uscx(Rest,H1),                         
                str_char(S,C),
		concat(S,H1,H2),!.

  filter_uscx("","").
    
  filter_usc([H|T],[Hn|L]):-
		filter_uscx(H,Hn),
		filter_usc(T,L),!.
  filter_usc([],[]).




                                                                 
