/*
===========================================================================
	DRAW.PRO   

	drawing routines for CHEMICAL project

	Copyright (c) 1986, 1990 by Prolog Development Center
===========================================================================
*/

code=3999
project "chemical"

include "chemglob.pro"
include "chemutil.pro"

/***************** some helps for drawing ********************************/


  /*** Special Characters ***/                            

PREDICATES
  vert_bar(char). horiz_bar(char).
  vert_db(char).  horiz_db(char).
  vert_trb(char). horiz_trb(char).
  slash(char). bk_slash(char).
          
CLAUSES
  vert_bar(Vb):-char_int(Vb,179).
  horiz_bar(Hb):-char_int(Hb,196).
    
  vert_db(Db):-char_int(Db,186).
  horiz_db(Db):-char_int(Db,205).
     
  vert_trb(Trb):-char_int(Trb,215).
  horiz_trb(Trb):-char_int(Trb,240).
  slash(Sl):-char_int(Sl,47). bk_slash(BS):-char_int(BS,92).
         
         
/*..................................................*/

PREDICATES
  center_r(integer).center_c(integer,integer,integer).
     
     
CLAUSES
  center_r(13).
  center_c(N,Gap,Col):- Col=(78-N-(N-1)*Gap) div 2.
        



        
 /**** Sub Drawing ******/

PREDICATES

  draw_chars_at(integer,integer,direction,char).
  draw_slash_atom_at(integer,integer,direction,symbol).
  draw_bk_slash_atom_at(integer,integer,direction,symbol).
  draw_db(int_list,integer,integer).
  draw_trb(int_list,integer,integer).



CLAUSES

  draw_chars_at(_,_,right(0),_):-!.

  draw_chars_at(Row,Col,right(N),C):-
   C11=Col+1,N1=N-1,
   scr_char(Row,C11,C),
   draw_chars_at(Row,C11,right(N1),C).


  draw_chars_at(Row,Col,left(N),C):-
   Cl=Col-N-1,
   draw_chars_at(Row,Cl,right(N),C).
   
  draw_chars_at(_,_,up(0),_):-!.

  draw_chars_at(Row,Col,up(N),C):-
   R1=Row-1,N1=N-1,
   scr_char(R1,Col,C),draw_chars_at(R1,Col,up(N1),C).

  draw_chars_at(_,_,down(0),_):-!.

  draw_chars_at(Row,Col,down(N),C):-
   R11=Row+1,N1=N-1,
   scr_char(R11,Col,C),draw_chars_at(R11,Col,down(N1),C).
   


  draw_slash_atom_at(Row,Col,right(_),S):-
   R1=Row-1,R2=Row-2,C11=Col+1,C22=Col+2,
   slash(Sl),scr_char(R1,C11,Sl),cursor(R2,C22),write(S),!.

  draw_slash_atom_at(Row,Col,left(_),S):-
   R11=Row+1,R22=Row+2,C1=Col-1,C2=Col-2,
   slash(Sl),scr_char(R11,C1,Sl),cursor(R22,C2),write(S),!.
   
  draw_bk_slash_atom_at(Row,Col,right(_),S):-
   R11=Row+1,R22=Row+2,C11=Col+1,C22=Col+2,
   bk_slash(Sl),scr_char(R11,C11,Sl),cursor(R22,C22),write(S),!.

  draw_bk_slash_atom_at(Row,Col,left(_),S):-
   R1=Row-1,R2=Row-2,C1=Col-1,C2=Col-2,
   bk_slash(Sl),scr_char(R1,C1,Sl),cursor(R2,C2),write(S),!.
   
  draw_db([],_,_).
  draw_db([1],2,Gap):-
   center_r(Row),center_c(2,Gap,Col),horiz_db(Db),    
   draw_chars_at(Row,Col,right(Gap),Db),
   draw_chars_at(Row,Col,up(2),' '),
   draw_chars_at(Row,Col,down(2),' '),
   draw_chars_at(Row,Col,left(3),' '),
   draw_bk_slash_atom_at(Row,Col,left(0),"H"),
   draw_slash_atom_at(Row,Col,left(0),"H"),
   draw_db([2],2,Gap),!.

  draw_db([1|D],N,Gap):-
   member(2,D,_),
   center_r(Row),center_c(N,Gap,Col),horiz_db(Db),    
   C11=Col+Gap+1,
   draw_chars_at(Row,Col,right(Gap),Db),
   draw_chars_at(Row,Col,up(2),' '),
   draw_chars_at(Row,Col,down(2),' '),
   draw_chars_at(Row,Col,left(3),' '),
   draw_bk_slash_atom_at(Row,Col,left(0),"H"),
   draw_slash_atom_at(Row,Col,left(0),"H"),
   draw_chars_at(Row,C11,down(2),' '),
   draw_db(D,N,Gap),!.

  draw_db([1|D],N,Gap):-
   center_r(Row),center_c(N,Gap,Col),horiz_db(Db),    
   C22=Col+Gap+1,
   draw_chars_at(Row,Col,right(Gap),Db),
   draw_chars_at(Row,Col,up(2),' '),
   draw_chars_at(Row,Col,down(2),' '),
   draw_chars_at(Row,Col,left(3),' '),
   draw_chars_at(Row,C22,up(2),' '),
   draw_bk_slash_atom_at(Row,Col,left(0),"H"),
   draw_slash_atom_at(Row,Col,left(0),"H"),
   draw_db(D,N,Gap),!.

  draw_db([N|D],N,Gap):-
   center_r(Row),center_c(N,Gap,Col),
   Ce=Col+(N-1)*(Gap+1),
   draw_chars_at(Row,Ce,up(2),' '),
   draw_chars_at(Row,Ce,down(2),' '),
   draw_chars_at(Row,Ce,right(3),' '),
   draw_slash_atom_at(Row,Ce,right(0),"H"),
   draw_bk_slash_atom_at(Row,Ce,right(0),"H"),
   draw_db(D,N,Gap),!.

   
  draw_db([I|D],N,Gap):-
   I11=I+1,member(I11,D,_),
   center_r(Row),center_c(N,Gap,Col),horiz_db(Db),    
   Cd=Col+(I-1)*(Gap+1),Ce=Col+I*(Gap+1),
   draw_chars_at(Row,Cd,right(Gap),Db),
   draw_chars_at(Row,Cd,up(2),' '),
   draw_chars_at(Row,Ce,down(2),' '),
   draw_db(D,N,Gap),!.

  draw_db([I|D],N,Gap):-
   I=N-1,
   center_r(Row),center_c(N,Gap,Col),horiz_db(Db),    
   Cd=Col+(I-1)*(Gap+1),
   draw_chars_at(Row,Cd,right(Gap),Db),
   draw_chars_at(Row,Cd,up(2),' '),
   draw_db([N|D],N,Gap),!.

  draw_db([I|D],N,Gap):-
   center_r(Row),center_c(N,Gap,Col),horiz_db(Db),    
   Cd=Col+(I-1)*(Gap+1),Cd11=Col+I*(Gap+1),
   draw_chars_at(Row,Cd,right(Gap),Db),
   draw_chars_at(Row,Cd,up(2),' '),
   draw_chars_at(Row,Cd11,up(2),' '),
   draw_db(D,N,Gap),!.



  draw_trb([],_,_):-!.
  draw_trb([I|D],N,Gap):-
   center_r(Row),center_c(N,Gap,Col),    
   Cd=Col+(I-1)*(Gap+1),Ce=Col+I*(Gap+1),
   horiz_trb(Tb),draw_chars_at(Row,Cd,right(Gap),Tb),
   draw_chars_at(Row,Cd,up(2),' '),
   draw_chars_at(Row,Cd,down(2),' '),
   draw_chars_at(Row,Ce,up(2),' '),
   draw_chars_at(Row,Ce,down(2),' '),
   draw_trb(D,N,Gap),!.


/**** Halogen ****/

PREDICATES
  draw_halogen_at(integer,integer,direction,symbol) 
  draw_halogens_sub(symbol,int_list,Llist,integer,int_list,int_list) 
  draw_halogens(Llist,Llist,integer,int_list) 


CLAUSES

  draw_halogen_at(Row,Col,right(_),H):-
   horiz_bar(Hb),C11=Col+1,C22=Col+2,
   scr_char(Row,C11,Hb),cursor(Row,C22),write(H),!.

  draw_halogen_at(Row,Col,left(_),H):-
   horiz_bar(Hb),C2=Col-2,C3=Col-3,
   cursor(Row,C2),write(Hb,Hb),cursor(Row,C3),write(H),!.
   
  draw_halogen_at(Row,Col,up(_),H):-
   vert_bar(Vb),R1=Row-1,R2=Row-2,
   scr_char(R1,Col,Vb),cursor(R2,Col),write(H),!.

  draw_halogen_at(Row,Col,down(_),H):-
   vert_bar(Vb),R11=Row+1,R22=Row+2,
   scr_char(R11,Col,Vb),cursor(R22,Col),write(H),!.
  
  draw_halogens_sub(_,[],_,_,SL,SL):-!.

  draw_halogens_sub(Fm,[1|IL],AEYne,Gap,StList,Nwlist):-
   member(trb(Tb),AEYne,_),member(1,Tb,_),
   member(nc(N),AEYne,_),
   center_r(Row),center_c(N,Gap,Col),
   draw_halogen_at(Row,Col,left(1),Fm),
   draw_halogens_sub(Fm,IL,AEYne,Gap,[1|StList],Nwlist),!.

  draw_halogens_sub(Fm,[1|IL],AEYne,Gap,StList,Nwlist):-
   member(db(Db),AEYne,_),member(1,Db,_),
   member(nc(N),AEYne,_),
   center_r(Row),center_c(N,Gap,Col),
   count_n(1,StList,Cnt),Cnt=0,
   draw_slash_atom_at(Row,Col,left(1),Fm),
   draw_halogens_sub(Fm,IL,AEYne,Gap,[1|StList],Nwlist),!.

  draw_halogens_sub(Fm,[1|IL],AEYne,Gap,StList,Nwlist):-
   member(db(Db),AEYne,_),member(1,Db,_),
   member(nc(N),AEYne,_),
   center_r(Row),center_c(N,Gap,Col),
   count_n(1,StList,Cnt),Cnt=1,
   draw_bk_slash_atom_at(Row,Col,left(1),Fm),
   draw_halogens_sub(Fm,IL,AEYne,Gap,[1|StList],Nwlist),!.

  draw_halogens_sub(Fm,[1|IL],AEYne,Gap,StList,Nwlist):-
   count_n(1,StList,Cnt),Cnt=2,
   member(nc(1),AEYne,_),
   center_r(Row),center_c(1,Gap,Col),
   draw_halogen_at(Row,Col,left(1),Fm),
   draw_halogens_sub(Fm,IL,AEYne,Gap,[1|StList],Nwlist),!.

  draw_halogens_sub(Fm,[I|IL],AEYne,Gap,StList,Nwlist):-
   member(nc(N),AEYne,_),I<N,
   count_n(I,StList,Cnt),Cnt=2,
   center_r(Row),center_c(N,Gap,Col),
   Ch=Col+(I-1)*(Gap+1),
   draw_halogen_at(Row,Ch,left(1),Fm),
   draw_halogens_sub(Fm,IL,AEYne,Gap,[I|StList],Nwlist),!.

  draw_halogens_sub(Fm,[1|IL],AEYne,Gap,StList,Nwlist):-
   member(nc(1),AEYne,_),
   count_n(1,StList,Cnt),Cnt=3,
   center_r(Row),center_c(1,Gap,Col),
   draw_halogen_at(Row,Col,right(1),Fm),
   draw_halogens_sub(Fm,IL,AEYne,Gap,[1|StList],Nwlist),!.
   
  draw_halogens_sub(Fm,[N|IL],AEYne,Gap,StList,Nwlist):-
   member(nc(N),AEYne,_),
   member(trb(Tb),AEYne,_),N1=N-1,member(N1,Tb,_),
   center_r(Row),center_c(N,Gap,Col),
   Ch=Col+N1*(Gap+1),
   draw_halogen_at(Row,Ch,right(1),Fm),
   draw_halogens_sub(Fm,IL,AEYne,Gap,[N|StList],Nwlist),!.

  draw_halogens_sub(Fm,[N|IL],AEYne,Gap,StList,Nwlist):-
   member(nc(N),AEYne,_),
   member(db(Db),AEYne,_),N1=N-1,member(N1,Db,_),
   center_r(Row),center_c(N,Gap,Col),
   count_n(N,StList,Cnt),Cnt=0,Ch=Col+N1*(Gap+1),
   draw_bk_slash_atom_at(Row,Ch,right(1),Fm),
   draw_halogens_sub(Fm,IL,AEYne,Gap,[N|StList],Nwlist),!.

  draw_halogens_sub(Fm,[N|IL],AEYne,Gap,StList,Nwlist):-
   member(nc(N),AEYne,_),
   member(db(Db),AEYne,_),N1=N-1,member(N1,Db,_),
   center_r(Row),center_c(N,Gap,Col),
   Ch=Col+N1*(Gap+1),
   count_n(N,StList,Cnt),Cnt=1,
   draw_slash_atom_at(Row,Ch,right(1),Fm),
   draw_halogens_sub(Fm,IL,AEYne,Gap,[N|StList],Nwlist),!.

  draw_halogens_sub(Fm,[N|IL],AEYne,Gap,StList,Nwlist):-
   member(nc(N),AEYne,_),
   count_n(N,StList,Cnt),Cnt=2,
   center_r(Row),center_c(N,Gap,Col),
   Ch=Col+(N-1)*(Gap+1),
   draw_halogen_at(Row,Ch,right(1),Fm),
   draw_halogens_sub(Fm,IL,AEYne,Gap,[N|StList],Nwlist),!.

  draw_halogens_sub(Fm,[I|IL],AEYne,Gap,StList,Nwlist):-
   member(nc(N),AEYne,_),
   center_r(Row),center_c(N,Gap,Col),
   Ch=Col+(I-1)*(Gap+1),
   count_n(I,StList,Cnt),Cnt=0,
   draw_halogen_at(Row,Ch,down(1),Fm),
   draw_halogens_sub(Fm,IL,AEYne,Gap,[I|StList],Nwlist),!.
   
  draw_halogens_sub(Fm,[I|IL],AEYne,Gap,StList,Nwlist):-
   member(nc(N),AEYne,_),
   center_r(Row),center_c(N,Gap,Col),
   Ch=Col+(I-1)*(Gap+1),
   count_n(I,StList,Cnt),Cnt=1,
   draw_halogen_at(Row,Ch,up(1),Fm),
   draw_halogens_sub(Fm,IL,AEYne,Gap,[I|StList],Nwlist),!.
   
  draw_halogens_sub(Fm,[I|IL],AEYne,Gap,StList,Nwlist):-
   member(nc(N),AEYne,_),
   center_r(Row),center_c(N,Gap,Col),
   Ch=Col+(I-1)*(Gap+1),
   count_n(I,StList,Cnt),Cnt=1,
   draw_halogen_at(Row,Ch,up(1),Fm),
   draw_halogens_sub(Fm,IL,AEYne,Gap,[I|StList],Nwlist),!.
 
 
  draw_halogens([],_,_,_):-!.
  draw_halogens([sp_name(Fm),loc(I)|List],AEYne,Gap,StList):-
   draw_halogens_sub(Fm,I,AEYne,Gap,StList,NwList),
   draw_halogens(List,AEYne,Gap,NwList),!.

/****   Alkyls  ****/

PREDICATES
  draw_ch2s_at(integer,integer,integer,direction) 
  draw_alkyl_at(integer,integer,direction) 
  draw_alkyls_sub(symbol,integer,int_list,integer,integer,int_list,int_list,int_list,integer,integer) 
  draw_alkyls(Llist,integer,integer,int_list,int_list,int_list,integer,integer) 


CLAUSES

  draw_ch2s_at(Row,Col,_,right(1)):-  
   vert_bar(Vb),  
   R2=Row-2,R1=Row-1,R11=Row+1,R22=Row+2,
   scr_char(R2,Col,'H'),scr_char(R1,Col,Vb),
   scr_char(Row,Col,'C'),
   scr_char(R11,Col,Vb),scr_char(R22,Col,'H'),!.

  draw_ch2s_at(Row,Col,Gap,right(N)):-
   vert_bar(Vb),horiz_bar(Hb), 
   R2=Row-2,R1=Row-1,R11=Row+1,R22=Row+2,
   scr_char(R2,Col,'H'),scr_char(R1,Col,Vb),
   scr_char(Row,Col,'C'),
   scr_char(R11,Col,Vb),scr_char(R22,Col,'H'),
   draw_chars_at(Row,Col,right(Gap),Hb),       
   Cpp=Col+Gap+1,N1=N-1,draw_ch2s_at(Row,Cpp,Gap,right(N1)),!.
   
  draw_ch2s_at(Row,Col,Gap,up(1)):-  
   horiz_bar(Hb),vert_bar(Vb),  
   Rp=Row-Gap,Rp1=Rp-1,C2=Col-2,C1=Col-1,C11=Col+1,C22=Col+2,
   draw_chars_at(Row,Col,up(Gap),Vb),
   scr_char(Rp1,Col,'C'),scr_char(Rp1,C2,'H'),scr_char(Rp1,C1,Hb),
   scr_char(Rp1,Col,'C'),scr_char(Rp1,C11,Hb),scr_char(Rp1,C22,'H'),!.
   

  draw_ch2s_at(Row,Col,Gap,up(N)):-
   vert_bar(Vb),horiz_bar(Hb), 
   Rp=Row-Gap,Rp1=Rp-1,
   C2=Col-2,C1=Col-1,C11=Col+1,C22=Col+2,
   draw_chars_at(Row,Col,up(Gap),Vb),
   scr_char(Rp1,C2,'H'),scr_char(Rp1,C1,Hb),
   scr_char(Rp1,Col,'C'),
   scr_char(Rp1,C11,Hb),scr_char(Rp1,C22,'H'),
   N1=N-1,draw_ch2s_at(Rp1,Col,Gap,up(N1)),!.

  draw_ch2s_at(Row,Col,Gap,down(1)):-  
   horiz_bar(Hb),vert_bar(Vb),  
   Rpp=Row+Gap,Rp11=Rpp+1,
   C2=Col-2,C1=Col-1,C11=Col+1,C22=Col+2,
   draw_chars_at(Row,Col,down(Gap),Vb),
   scr_char(Rp11,Col,'C'),scr_char(Rp11,C2,'H'),scr_char(Rp11,C1,Hb),
   scr_char(Rp11,Col,'C'),scr_char(Rp11,C11,Hb),scr_char(Rp11,C22,'H'),!.
   

  draw_ch2s_at(Row,Col,Gap,down(N)):-
   vert_bar(Vb),horiz_bar(Hb), 
   Rpp=Row+Gap,Rp11=Rpp+1,
   C2=Col-2,C1=Col-1,C11=Col+1,C22=Col+2,
   draw_chars_at(Row,Col,down(Gap),Vb),
   scr_char(Rp11,C2,'H'),scr_char(Rp11,C1,Hb),
   scr_char(Rp11,Col,'C'),
   scr_char(Rp11,C11,Hb),scr_char(Rp11,C22,'H'),
   N1=N-1,draw_ch2s_at(Rp11,Col,Gap,down(N1)),!.

  draw_alkyl_at(Row,Col,up(N)):-
   draw_ch2s_at(Row,Col,1,up(N)),
   Rb=Row-(2*N+1),Rh=Rb-1,vert_bar(Vb),
   scr_char(Rb,Col,Vb),scr_char(Rh,Col,'H'),!.

  draw_alkyl_at(Row,Col,down(N)):-
   draw_ch2s_at(Row,Col,1,down(N)),
   Rbb=Row+(2*N+1),Rhh=Rbb+1,vert_bar(Vb),
   scr_char(Rbb,Col,Vb),scr_char(Rhh,Col,'H'),!.
  
  draw_alkyls_sub(_,_,[],_,_,_,Sl,Sl,Fl,Fl):-!.

  draw_alkyls_sub(iso,Na,[I|IL],Nc,Gap,Db_list,StList,NwList,_,NwFlag):-
   member(I,Db_list,_),N2=Na-2,
   center_r(Row),center_c(Nc,Gap,Col),
   Ca=Col+(I-1)*(Gap+1),draw_alkyl_at(Row,Ca,up(N2)),
   Rc=Row-2*N2 ,Rr=Rc-2,Ca2=Ca-2,
   scr_char(Rr,Ca,'R'),scr_char(Rc,Ca2,'R'),
   draw_chars_at(Row,Ca,down(2),' '), 
   draw_alkyls_sub(iso,Na,IL,Nc,Gap,Db_list,[I|StList],NwList,1,NwFlag),!.

  draw_alkyls_sub(iso,Na,[I|IL],Nc,Gap,Db_list,StList,NwList,_,NwFlag):-
   count_n(I,StList,Cnt),Cnt=0,N2=Na-2,
   center_r(Row),center_c(Nc,Gap,Col),
   Ca=Col+(I-1)*(Gap+1),draw_alkyl_at(Row,Ca,up(N2)),
   Rc=Row-2*N2 ,Rr=Rc-2,Ca2=Ca-2,
   scr_char(Rr,Ca,'R'),scr_char(Rc,Ca2,'R'),
   draw_alkyls_sub(iso,Na,IL,Nc,Gap,Db_list,[I|StList],NwList,1,NwFlag),!.

  draw_alkyls_sub(iso,Na,[I|IL],Nc,Gap,Db_list,StList,NwList,_,NwFlag):-
   count_n(I,StList,Cnt),Cnt=1,N2=Na-2,
   center_r(Row),center_c(Nc,Gap,Col),
   Ca=Col+(I-1)*(Gap+1),draw_alkyl_at(Row,Ca,down(N2)),
   Rc=Row+2*N2,Rr=Rc+2,Ca2=Ca-2,
   scr_char(Rr,Ca,'R'),scr_char(Rc,Ca2,'R'),
   draw_alkyls_sub(iso,Na,IL,Nc,Gap,Db_list,[I|StList],NwList,1,NwFlag),!.

  draw_alkyls_sub(neo,Na,[I|IL],Nc,Gap,Db_list,StList,NwList,_,NwFlag):-
   member(I,Db_list,_),N3=Na-3,
   center_r(Row),center_c(Nc,Gap,Col),
   Ca=Col+(I-1)*(Gap+1),draw_alkyl_at(Row,Ca,up(N3)),
   Rc=Row-2*N3, Rr=Rc-2,Ca2=Ca-2,Ca22=Ca+2,
   scr_char(Rr,Ca,'R'),scr_char(Rc,Ca2,'R'),scr_char(Rc,Ca22,'R'),
   draw_chars_at(Row,Ca,down(2),' '),
   draw_alkyls_sub(neo,Na,IL,Nc,Gap,Db_list,[I|StList],NwList,1,NwFlag),!.

  draw_alkyls_sub(neo,Na,[I|IL],Nc,Gap,Db_list,StList,NwList,_,NwFlag):-
   count_n(I,StList,Cnt),Cnt=0,N3=Na-3,
   center_r(Row),center_c(Nc,Gap,Col),
   Ca=Col+(I-1)*(Gap+1),draw_alkyl_at(Row,Ca,up(N3)),
   Rc=Row-2*N3, Rr=Rc-2,Ca2=Ca-2,Ca22=Ca+2,
   scr_char(Rr,Ca,'R'),scr_char(Rc,Ca2,'R'),scr_char(Rc,Ca22,'R'),
   draw_alkyls_sub(neo,Na,IL,Nc,Gap,Db_list,[I|StList],NwList,1,NwFlag),!.

  draw_alkyls_sub(neo,Na,[I|IL],Nc,Gap,Db_list,StList,NwList,_,NwFlag):-
   count_n(I,StList,Cnt),Cnt=1,N3=Na-3,
   center_r(Row),center_c(Nc,Gap,Col),
   Ca=Col+(I-1)*(Gap+1),draw_alkyl_at(Row,Ca,down(N3)),
   Rc=Row+2*N3,Rr=Rc+2,Ca2=Ca-2,Ca22=Ca+2,
   scr_char(Rr,Ca,'R'),scr_char(Rc,Ca2,'R'),scr_char(Rc,Ca22,'R'),
   draw_alkyls_sub(neo,Na,IL,Nc,Gap,Db_list,[I|StList],NwList,1,NwFlag),!.

  draw_alkyls_sub(sec,Na,[I|IL],Nc,Gap,Db_list,StList,NwList,_,NwFlag):-
   member(I,Db_list,_),N1=Na-1,
   center_r(Row),center_c(Nc,Gap,Col),
   Ca=Col+(I-1)*(Gap+1),draw_alkyl_at(Row,Ca,up(N1)),
   Rc=Row-2,Ca2=Ca-2,
   scr_char(Rc,Ca2,'R'),draw_chars_at(Row,Ca,down(2),' '),
   draw_alkyls_sub(sec,Na,IL,Nc,Gap,Db_list,[I|StList],NwList,1,NwFlag),!.
  
  draw_alkyls_sub(sec,Na,[I|IL],Nc,Gap,Db_list,StList,NwList,_,NwFlag):-
   count_n(I,StList,Cnt),Cnt=0,N1=Na-1,
   center_r(Row),center_c(Nc,Gap,Col),
   Ca=Col+(I-1)*(Gap+1),draw_alkyl_at(Row,Ca,up(N1)),
   Rc=Row-2,Ca2=Ca-2,
   scr_char(Rc,Ca2,'R'),
   draw_alkyls_sub(sec,Na,IL,Nc,Gap,Db_list,[I|StList],NwList,1,NwFlag),!.

  draw_alkyls_sub(sec,Na,[I|IL],Nc,Gap,Db_list,StList,NwList,_,NwFlag):-
   count_n(I,StList,Cnt),Cnt=1,N1=Na-1,
   center_r(Row),center_c(Nc,Gap,Col),
   Ca=Col+(I-1)*(Gap+1),draw_alkyl_at(Row,Ca,down(N1)),
   Rc=Row+2,Ca2=Ca-2,
   scr_char(Rc,Ca2,'R'),
   draw_alkyls_sub(sec,Na,IL,Nc,Gap,Db_list,[I|StList],NwList,1,NwFlag),!.

  draw_alkyls_sub(tert,Na,[I|IL],Nc,Gap,Db_list,StList,NwList,_,NwFlag):-
   member(I,Db_list,_),N2=Na-2,
   center_r(Row),center_c(Nc,Gap,Col),
   Ca=Col+(I-1)*(Gap+1),draw_alkyl_at(Row,Ca,up(N2)),
   Rc=Row-2,Ca2=Ca-2,Ca22=Ca+2,
   scr_char(Rc,Ca2,'R'),scr_char(Rc,Ca22,'R'),
   draw_chars_at(Row,Ca,down(2),' '),
   draw_alkyls_sub(tert,Na,IL,Nc,Gap,Db_list,[I|StList],NwList,1,NwFlag),!.
 
  draw_alkyls_sub(tert,Na,[I|IL],Nc,Gap,Db_list,StList,NwList,_,NwFlag):-
   count_n(I,StList,Cnt),Cnt=0,N2=Na-2,
   center_r(Row),center_c(Nc,Gap,Col),
   Ca=Col+(I-1)*(Gap+1),draw_alkyl_at(Row,Ca,up(N2)),
   Rc=Row-2,Ca2=Ca-2,Ca22=Ca+2,
   scr_char(Rc,Ca2,'R'),scr_char(Rc,Ca22,'R'),
   draw_alkyls_sub(tert,Na,IL,Nc,Gap,Db_list,[I|StList],NwList,1,NwFlag),!.

  draw_alkyls_sub(tert,Na,[I|IL],Nc,Gap,Db_list,StList,NwList,_,NwFlag):-
   count_n(I,StList,Cnt),Cnt=1,N2=Na-2,
   center_r(Row),center_c(Nc,Gap,Col),
   Ca=Col+(I-1)*(Gap+1),draw_alkyl_at(Row,Ca,down(N2)),
   Rc=Row+2,Ca2=Ca-2,Ca22=Ca+2,
   scr_char(Rc,Ca2,'R'),scr_char(Rc,Ca22,'R'),
   draw_alkyls_sub(tert,Na,IL,Nc,Gap,Db_list,[I|StList],NwList,1,NwFlag),!.

  draw_alkyls_sub("",Na,[I|IL],Nc,Gap,Db_list,StList,NwList,Flag,NwFlag):-
   member(I,Db_list,_),
   center_r(Row),center_c(Nc,Gap,Col),
   Ca=Col+(I-1)*(Gap+1),draw_alkyl_at(Row,Ca,up(Na)),
   draw_chars_at(Row,Ca,down(2),' '),
   draw_alkyls_sub("",Na,IL,Nc,Gap,Db_list,[I|StList],NwList,Flag,NwFlag),!.

  draw_alkyls_sub("",Na,[I|IL],Nc,Gap,Db_list,StList,NwList,Flag,NwFlag):-
   count_n(I,StList,Cnt),Cnt=0,
   center_r(Row),center_c(Nc,Gap,Col),
   Ca=Col+(I-1)*(Gap+1),draw_alkyl_at(Row,Ca,up(Na)),
   draw_alkyls_sub("",Na,IL,Nc,Gap,Db_list,[I|StList],NwList,Flag,NwFlag),!.

  draw_alkyls_sub("",Na,[I|IL],Nc,Gap,Db_list,StList,NwList,Flag,NwFlag):-
   count_n(I,StList,Cnt),Cnt=1,
   center_r(Row),center_c(Nc,Gap,Col),
   Ca=Col+(I-1)*(Gap+1),draw_alkyl_at(Row,Ca,down(Na)),
   draw_alkyls_sub("",Na,IL,Nc,Gap,Db_list,[I|StList],NwList,Flag,NwFlag),!.

  draw_alkyls([],_,_,_,Sl,Sl,Fl,Fl):-!.
  draw_alkyls([sp_name(Nm),nc(Na),loc(I)|List],Nc,Gap,Db_list,StL,NL,FL,NF):-
   draw_alkyls_sub(Nm,Na,I,Nc,Gap,Db_list,StL,NLx,FL,NFx),
   draw_alkyls(List,Nc,Gap,Db_list,NLx,NL,NFx,NF),!.


/**** Alkenynyl ****/   

PREDICATES
  draw_alk_db(integer,integer,int_list,direction) 
  draw_alk_trb(integer,integer,int_list,direction) 
  draw_spcl_yls(Llist,integer,integer,int_list,int_list,int_list,integer,integer) 
  draw_spcl_yls_sub(symbol,int_list,integer,integer,int_list,int_list,int_list,integer,integer) 
  draw_enyls(Llist,integer,integer,int_list,int_list,int_list) 
  draw_enyls_sub(integer,int_list,int_list,int_list,integer,integer,int_list,int_list,int_list) 
 
CLAUSES
 
  draw_alk_db(_,_,[],_):-!.

  draw_alk_db(Row,Col,[N1|IL],up(N)):-
   N1=N-1,Rc=Row-2*N1,vert_db(Db),
   draw_chars_at(Rc,Col,up(1),Db),
   draw_chars_at(Rc,Col,left(2),' '),
   draw_alk_db(Row,Col,[N|IL],up(N)),!.

  draw_alk_db(Row,Col,[N|IL],up(N)):-
  Rc=Row-2*N,
   draw_chars_at(Rc,Col,right(2),' '),
   draw_chars_at(Rc,Col,left(2),' '),
   draw_chars_at(Rc,Col,up(2),' '),
   draw_slash_atom_at(Rc,Col,right(0),"H"),
   draw_bk_slash_atom_at(Rc,Col,left(0),"H"),
   draw_alk_db(Row,Col,IL,up(N)),!.

  draw_alk_db(Row,Col,[I|IL],up(N)):-
   I11=I+1,member(I11,IL,_),
   RcI=Row-2*I,RcI11=Row-2*I11,
   vert_db(Db),
   draw_chars_at(RcI,Col,up(1),Db),
   draw_chars_at(RcI,Col,left(2),' '),
   draw_chars_at(RcI11,Col,right(2),' '),
   draw_alk_db(Row,Col,IL,up(N)),!.

  draw_alk_db(Row,Col,[I|IL],up(N)):-
   I11=I+1,RcI=Row-2*I,RcI11=Row-2*I11,
   vert_db(Db),
   draw_chars_at(RcI,Col,up(1),Db),
   draw_chars_at(RcI,Col,left(2),' '),
   draw_chars_at(RcI11,Col,left(2),' '),
   draw_alk_db(Row,Col,IL,up(N)),!.

  draw_alk_db(Row,Col,[N1|IL],down(N)):-
   N1=N-1,Rc=Row+2*N1,vert_db(Db),
   draw_chars_at(Rc,Col,down(1),Db),
   draw_chars_at(Rc,Col,left(2),' '),
   draw_alk_db(Row,Col,[N|IL],down(N)),!.

  draw_alk_db(Row,Col,[N|IL],down(N)):-
   Rc=Row+2*N,
   draw_chars_at(Rc,Col,right(2),' '),
   draw_chars_at(Rc,Col,left(2),' '),
   draw_chars_at(Rc,Col,down(2),' '),
   draw_slash_atom_at(Rc,Col,left(0),"H"),
   draw_bk_slash_atom_at(Rc,Col,right(0),"H"),
   draw_alk_db(Row,Col,IL,down(N)),!.

  draw_alk_db(Row,Col,[I|IL],down(N)):-
   I11=I+1,member(I11,IL,_),
   RcI=Row+2*I,RcI11=Row+2*I11,
   vert_db(Db),
   draw_chars_at(RcI,Col,down(1),Db),
   draw_chars_at(RcI,Col,left(2),' '),
   draw_chars_at(RcI11,Col,right(2),' '),
   draw_alk_db(Row,Col,IL,down(N)),!.

  draw_alk_db(Row,Col,[I|IL],down(N)):-
   I11=I+1,RcI=Row+2*I,RcI11=Row+2*I11,
   vert_db(Db),
   draw_chars_at(RcI,Col,down(1),Db),
   draw_chars_at(RcI,Col,left(2),' '),
   draw_chars_at(RcI11,Col,left(2),' '),
   draw_alk_db(Row,Col,IL,down(N)),!.

  draw_alk_trb(_,_,[],_):-!.
  draw_alk_trb(Row,Col,[I|T],up(N)):-
   Rc=Row-2*I,Rc11=Row-2*(I+1),vert_trb(Trb),
   draw_chars_at(Rc,Col,up(1),Trb),
   draw_chars_at(Rc,Col,left(2),' '),
   draw_chars_at(Rc,Col,right(2),' '),
   draw_chars_at(Rc11,Col,left(2),' '),
   draw_chars_at(Rc11,Col,right(2),' '),
   draw_alk_trb(Row,Col,T,up(N)),!.

  draw_alk_trb(Row,Col,[I|T],down(N)):-
   Rc=Row+2*I,Rc11=Row+2*(I+1),vert_trb(Trb),
   draw_chars_at(Rc,Col,down(1),Trb),
   draw_chars_at(Rc,Col,left(2),' '),
   draw_chars_at(Rc,Col,right(2),' '),
   draw_chars_at(Rc11,Col,left(2),' '),
   draw_chars_at(Rc11,Col,right(2),' '),
   draw_alk_trb(Row,Col,T,down(N)),!.
   
   
   
  draw_spcl_yls_sub(_,[],_,_,_,Sl,Sl,Fl,Fl):-!.
  draw_spcl_yls_sub(isopropen,[I|IL],Nc,Gap,Db_list,StList,NwList,_,NwFlag):-
   member(I,Db_list,_),
   center_r(Row),center_c(Nc,Gap,Col),
   Ca=Col+(I-1)*(Gap+1),draw_alkyl_at(Row,Ca,up(2)),
   draw_alk_db(Row,Ca,[1],up(2)), 
   R2=Row-2,Ca22=Ca+2,
   draw_chars_at(R2,Ca,left(2),' '),scr_char(R2,Ca22,'R'),
   draw_chars_at(Row,Ca,down(2),' '),
   draw_spcl_yls_sub(isopropen,IL,Nc,Gap,Db_list,[I|StList],NwList,1,NwFlag),!.
 
  draw_spcl_yls_sub(isopropen,[I|IL],Nc,Gap,Db_list,StList,NwList,_,NwFlag):-
   count_n(I,StList,Cnt),Cnt=0,
   center_r(Row),center_c(Nc,Gap,Col),
   Ca=Col+(I-1)*(Gap+1),draw_alkyl_at(Row,Ca,up(2)),
   draw_alk_db(Row,Ca,[1],up(2)), 
   R2=Row-2,Ca22=Ca+2,
   draw_chars_at(R2,Ca,left(2),' '),scr_char(R2,Ca22,'R'),
   draw_spcl_yls_sub(isopropen,IL,Nc,Gap,Db_list,[I|StList],NwList,1,NwFlag),!.

  draw_spcl_yls_sub(isopropen,[I|IL],Nc,Gap,Db_list,StList,NwList,_,NwFlag):-
   count_n(I,StList,Cnt),Cnt=1,
   center_r(Row),center_c(Nc,Gap,Col),
   Ca=Col+(I-1)*(Gap+1),draw_alkyl_at(Row,Ca,down(2)),
   draw_alk_db(Row,Ca,[1],down(2)), 
   R22=Row+2,Ca22=Ca+2,
   draw_chars_at(R22,Ca,left(2),' '),scr_char(R22,Ca22,'R'),
   draw_spcl_yls_sub(isopropen,IL,Nc,Gap,Db_list,[I|StList],NwList,1,NwFlag),!.

  draw_spcl_yls_sub(Nm,[I|IL],Nc,Gap,Db_list,StList,NwList,Flag,NwFlag):-
   member(I,Db_list,_),
   center_r(Row),center_c(Nc,Gap,Col),
   Ca=Col+(I-1)*(Gap+1),
   sp_stem(Nm,N),draw_alkyl_at(Row,Ca,up(N)),
   N1=N-1,draw_alk_db(Row,Ca,[N1],up(N)), 
   R2=Row-2*N1,
   draw_chars_at(R2,Ca,left(2),' '),draw_chars_at(Row,Ca,down(2),' '),
   draw_spcl_yls_sub(Nm,IL,Nc,Gap,Db_list,[I|StList],NwList,Flag,NwFlag),!.
 
  draw_spcl_yls_sub(Nm,[I|IL],Nc,Gap,Db_list,StList,NwList,Flag,NwFlag):-
   count_n(I,StList,Cnt),Cnt=0,
   center_r(Row),center_c(Nc,Gap,Col),
   Ca=Col+(I-1)*(Gap+1),
   sp_stem(Nm,N),draw_alkyl_at(Row,Ca,up(N)),
   N1=N-1,draw_alk_db(Row,Ca,[N1],up(N)), 
   R2=Row-2*N1,
   draw_chars_at(R2,Ca,left(2),' '),
   draw_spcl_yls_sub(Nm,IL,Nc,Gap,Db_list,[I|StList],NwList,Flag,NwFlag),!.

  draw_spcl_yls_sub(Nm,[I|IL],Nc,Gap,Db_list,StList,NwList,Flag,NwFlag):-
   count_n(I,StList,Cnt),Cnt=1,
   center_r(Row),center_c(Nc,Gap,Col),
   Ca=Col+(I-1)*(Gap+1),
   sp_stem(Nm,N),draw_alkyl_at(Row,Ca,down(N)),
   N1=N-1,draw_alk_db(Row,Ca,[N1],down(N)), 
   R2=Row+2*N1,
   draw_chars_at(R2,Ca,left(2),' '),
   draw_spcl_yls_sub(Nm,IL,Nc,Gap,Db_list,[I|StList],NwList,Flag,NwFlag),!.

  draw_spcl_yls([],_,_,_,Sl,Sl,Fl,Fl):-!.
  draw_spcl_yls([sp_name(Nm),loc(I)|List],Nc,Gap,Db_list,StList,NwList,Flag,NwFlag):-
   draw_spcl_yls_sub(Nm,I,Nc,Gap,Db_list,StList,NwL,Flag,NwFl),
   draw_spcl_yls(List,Nc,Gap,Db_list,NwL,NwList,NwFl,NwFlag),!.
 
   
 draw_enyls_sub(_,_,_,[],_,_,_,Sl,Sl):-!.

 draw_enyls_sub(Na,D,T,[I|IL],Nc,Gap,Db_list,StList,NwList):-
  member(I,Db_list,_),
  center_r(Row),center_c(Nc,Gap,Col),
  Ca=Col+(I-1)*(Gap+1),draw_alkyl_at(Row,Ca,up(Na)),
  draw_alk_db(Row,Ca,D,up(Na)),
  draw_alk_trb(Row,Ca,T,up(Na)),
  draw_chars_at(Row,Ca,down(2),' '),
  draw_enyls_sub(Na,D,T,IL,Nc,Gap,Db_list,[I|StList],NwList),!.

 draw_enyls_sub(Na,D,T,[I|IL],Nc,Gap,Db_list,StList,NwList):-
  count_n(I,StList,Cnt),Cnt=0,
  center_r(Row),center_c(Nc,Gap,Col),
  Ca=Col+(I-1)*(Gap+1),draw_alkyl_at(Row,Ca,up(Na)),
  draw_alk_db(Row,Ca,D,up(Na)),
  draw_alk_trb(Row,Ca,T,up(Na)),
  draw_enyls_sub(Na,D,T,IL,Nc,Gap,Db_list,[I|StList],NwList),!.
  
 draw_enyls_sub(Na,D,T,[I|IL],Nc,Gap,Db_list,StList,NwList):-
  count_n(I,StList,Cnt),Cnt=1,
  center_r(Row),center_c(Nc,Gap,Col),
  Ca=Col+(I-1)*(Gap+1),draw_alkyl_at(Row,Ca,down(Na)),
  draw_alk_db(Row,Ca,D,down(Na)),
  draw_alk_trb(Row,Ca,T,down(Na)),
  draw_enyls_sub(Na,D,T,IL,Nc,Gap,Db_list,[I|StList],NwList),!.
  
 draw_enyls([],_,_,_,Sl,Sl):-!.
 draw_enyls([nc(Na),db(D),trb(T),loc(I)|List],Nc,Gap,Db_list,StList,NwList):-
  draw_enyls_sub(Na,D,T,I,Nc,Gap,Db_list,StList,NwL),
   draw_enyls(List,Nc,Gap,Db_list,NwL,NwList),!.


/****  main drawing ****/

PREDICATES
  draw_aeyne(Llist,integer,integer) 


CLAUSES

  draw_aeyne([sp_name(""),nc(N),db(D),trb(T)],Gap,0):-
   center_r(Row),center_c(N,Gap,Col),horiz_bar(Hb),
   draw_ch2s_at(Row,Col,Gap,right(N)),
   Ce11=Col+(N-1)*(Gap+1)+1,C3=Col-3,
   cursor(Row,C3),write('H',Hb,Hb),
   cursor(Row,Ce11),write(Hb,Hb,'H'),
   draw_db(D,N,Gap),draw_trb(T,N,Gap),!.

  draw_aeyne([sp_name(iso),nc(N),_,_],Gap,1):-
   N2=N-2,center_r(Row),center_c(N2,Gap,Col),horiz_bar(Hb),
   draw_ch2s_at(Row,Col,Gap,right(N2)),
   Ce11=Col+(N2-1)*(Gap+1)+1,C3=Col-3,R2=Row-2,
   cursor(Row,C3),write('R',Hb,Hb),
   cursor(R2,Col),write('R'),
   cursor(Row,Ce11),write(Hb,Hb,'H'),!.

  draw_aeyne([sp_name(neo),nc(N),_,_],Gap,1):-
   N3=N-3,center_r(Row),center_c(N3,Gap,Col),horiz_bar(Hb),
   draw_ch2s_at(Row,Col,Gap,right(N3)),
   Ce11=Col+(N3-1)*(Gap+1)+1,C3=Col-3,R2=Row-2,R22=Row+2,
   cursor(Row,C3),write('R',Hb,Hb),
   cursor(R2,Col),write('R'),
   cursor(R22,Col),write('R'),
   cursor(Row,Ce11),write(Hb,Hb,'H'),!.
   
  draw([],[],[],[],H,[sp_name(Nm),nc(N),db(D),trb(T)],Flag):-
   gap_wob(Gap), sort(D,Ds),sort(T,Ts),
   draw_aeyne([sp_name(Nm),nc(N),db(Ds),trb(Ts)],Gap,Flag),
   draw_halogens(H,[sp_name(Nm),nc(N),db(Ds),trb(Ts)],Gap,[]),!.

  draw(Enynyl,Enyl,Spcl,Yl,H,[sp_name(Nm),nc(N),db(D),trb(T)],Flag):-
   gap_wb(Gap),sort(D,Ds),sort(T,Ts),make_db_trb_list(Ds,Dsx),
   draw_aeyne([sp_name(Nm),nc(N),db(Ds),trb(Ts)],Gap,Flag1),
   draw_halogens(H,[sp_name(Nm),nc(N),db(Ds),trb(Ts)],Gap,[]),
   draw_alkyls(Yl,N,Gap,Dsx,[],NwL1,Flag1,Flag2),
   draw_spcl_yls(Spcl,N,Gap,Dsx,NwL1,NwL2,Flag2,Flag),
   draw_enyls(Enyl,N,Gap,Dsx,NwL2,NwL3),
   draw_enyls(Enynyl,N,Gap,Dsx,NwL3,_),!.
                             
